import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:grinder/grinder.dart';
import 'package:http/http.dart' as http;
import 'package:meta/meta.dart';
import 'package:path/path.dart' as path;

part 'src/credentials.dart';

/// Checks whether the currently published version is older than this version,
/// and automatically publishes it.
Future<Null> autoPublish(
  final String packageName,
  final Credentials credentials,
) async {
  if (!await _shouldPublish(packageName)) {
    log('Not publishing.');
    return;
  }

  log("Writing the credentials file...");

  final pubCacheDir =
      new Directory(path.join(Platform.environment['HOME'], '.pub-cache'));

  // Create the pub-cache directory for the key.
  await pubCacheDir.create(recursive: true);

  // Write the credentials file.
  await new File(path.join(pubCacheDir.path, 'credentials.json'))
      .writeAsString(credentials.toJson());

  // Publish the package.
  run('pub', arguments: ['publish', '-f']);
}

Future<bool> _shouldPublish(String packageName) async {
  log('Comparing versions with pub now, to see if we should publish...');

  // Get the latest version for this package.
  // That's not the prettiest way to do it, but the package pub_client is
  // not ported to dart 2.0 yet.
  final String publishedVersion = (jsonDecode(
      (await http.get('https://pub.dartlang.org/api/packages/${packageName}'))
          .body) as Map)['latest']['version'];

  final pubspec = await new File('pubspec.yaml').readAsString();
  final version =
      new RegExp(r'^version:\s*(.*)$', multiLine: true, caseSensitive: false)
          .firstMatch(pubspec)
          .group(1);
  if (version != publishedVersion) {
    log('Publish the package! '
        'This version is $version and pub has $publishedVersion');
    return true;
  } else {
    log('Do not publish. Both versions are $version.');
    return false;
  }
}
